@@ -1269,7 +1269,7 @@ process_sf_dir(
 			namelen = ino_dir_size - ((char *)&sfep->name[0] -
 					 (char *)sfp);
 		} else if ((char *)sfep - (char *)sfp +
-				M_DIROPS(mp)->sf_entsize(sfp, sfep->namelen) >
+				libxfs_dir2_sf_entsize(mp, sfp, sfep->namelen) >
 				ino_dir_size) {
 			if (show_warnings)
 				print_warning("entry length in dir inode %llu "
@@ -1282,11 +1282,11 @@ process_sf_dir(
 
 		if (obfuscate)
 			generate_obfuscated_name(
-					 M_DIROPS(mp)->sf_get_ino(sfp, sfep),
+					 libxfs_dir2_sf_get_ino(mp, sfp, sfep),
 					 namelen, &sfep->name[0]);
 
 		sfep = (xfs_dir2_sf_entry_t *)((char *)sfep +
-				M_DIROPS(mp)->sf_entsize(sfp, namelen));
+				libxfs_dir2_sf_entsize(mp, sfp, namelen));
 	}
 
 	/* zero stale data in rest of space in data fork, if any */
@@ -1433,7 +1433,7 @@ process_dir_free_block(
 		return;
 
 	free = (struct xfs_dir2_free *)block;
-	M_DIROPS(mp)->free_hdr_from_disk(&freehdr, free);
+	libxfs_dir2_free_hdr_from_disk(mp, &freehdr, free);
 
 	switch (freehdr.magic) {
 	case XFS_DIR2_FREE_MAGIC:
@@ -1443,7 +1443,7 @@ process_dir_free_block(
 		int			used;
 
 		/* Zero out space from end of bests[] to end of block */
-		bests = M_DIROPS(mp)->free_bests_p(free);
+		bests = freehdr.bests;
 		high = (char *)&bests[freehdr.nvalid];
 		used = high - (char*)free;
 		memset(high, 0, mp->m_dir_geo->blksize - used);
@@ -1471,19 +1471,17 @@ process_dir_leaf_block(
 
 	/* Yes, this works for dir2 & dir3.  Difference is padding. */
 	leaf = (struct xfs_dir2_leaf *)block;
-	M_DIROPS(mp)->leaf_hdr_from_disk(&leafhdr, leaf);
+	libxfs_dir2_leaf_hdr_from_disk(mp, &leafhdr, leaf);
 
 	switch (leafhdr.magic) {
 	case XFS_DIR2_LEAF1_MAGIC:
 	case XFS_DIR3_LEAF1_MAGIC: {
 		struct xfs_dir2_leaf_tail	*ltp;
 		__be16				*lbp;
-		struct xfs_dir2_leaf_entry	*ents;
 		char				*free; /* end of ents */
 
 		/* Zero out space from end of ents[] to bests */
-		ents = M_DIROPS(mp)->leaf_ents_p(leaf);
-		free = (char *)&ents[leafhdr.count];
+		free = (char *)&leafhdr.ents[leafhdr.count];
 		ltp = xfs_dir2_leaf_tail_p(mp->m_dir_geo, leaf);
 		lbp = xfs_dir2_leaf_bests_p(ltp);
 		memset(free, 0, (char *)lbp - free);
@@ -1492,13 +1490,11 @@ process_dir_leaf_block(
 	}
 	case XFS_DIR2_LEAFN_MAGIC:
 	case XFS_DIR3_LEAFN_MAGIC: {
-		struct xfs_dir2_leaf_entry	*ents;
 		char				*free;
 		int				used;
 
 		/* Zero out space from end of ents[] to end of block */
-		ents = M_DIROPS(mp)->leaf_ents_p(leaf);
-		free = (char *)&ents[leafhdr.count];
+		free = (char *)&leafhdr.ents[leafhdr.count];
 		used = free - (char*)leaf;
 		memset(free, 0, mp->m_dir_geo->blksize - used);
 		iocur_top->need_crc = 1;
@@ -1560,7 +1556,7 @@ process_dir_data_block(
 		return;
 	}
 
-	dir_offset = M_DIROPS(mp)->data_entry_offset;
+	dir_offset = mp->m_dir_geo->data_entry_offset;
 	ptr = block + dir_offset;
 	endptr = block + mp->m_dir_geo->blksize;
 
@@ -1606,7 +1602,7 @@ process_dir_data_block(
 		}
 
 		dep = (xfs_dir2_data_entry_t *)ptr;
-		length = M_DIROPS(mp)->data_entsize(dep->namelen);
+		length = libxfs_dir2_data_entsize(mp, dep->namelen);
 
 		if (dir_offset + length > end_of_data ||
 		    ptr + length > endptr) {
@@ -1616,7 +1612,7 @@ process_dir_data_block(
 					(long long)cur_ino);
 			return;
 		}
-		if (be16_to_cpu(*M_DIROPS(mp)->data_entry_tag_p(dep)) !=
+		if (be16_to_cpu(*libxfs_dir2_data_entry_tag_p(mp, dep)) !=
 				dir_offset)
 			return;
 
@@ -1629,7 +1625,7 @@ process_dir_data_block(
 		if (zero_stale_data) {
 			/* 1 byte for ftype; don't bother with conditional */
 			int zlen =
-				(char *)M_DIROPS(mp)->data_entry_tag_p(dep) -
+				(char *)libxfs_dir2_data_entry_tag_p(mp, dep) -
 				(char *)&dep->name[dep->namelen] - 1;
 			if (zlen > 0) {
 				memset(&dep->name[dep->namelen] + 1, 0, zlen);
@@ -1881,8 +1877,19 @@ process_single_fsb_objects(
 				struct xfs_da3_icnode_hdr hdr;
 				int used;
 
-				M_DIROPS(mp)->node_hdr_from_disk(&hdr, node);
-				used = M_DIROPS(mp)->node_hdr_size;
+				libxfs_da3_node_hdr_from_disk(mp, &hdr, node);
+				switch (btype) {
+				case TYP_DIR2:
+					used = mp->m_dir_geo->node_hdr_size;
+					break;
+				case TYP_ATTR:
+					used = mp->m_attr_geo->node_hdr_size;
+					break;
+				default:
+					/* unknown type, don't zero anything */
+					used = mp->m_sb.sb_blocksize;
+					break;
+				}
 
 				used += hdr.count
 					* sizeof(struct xfs_da_node_entry);
